# Linear regression - single input x. Sampling distributions conditional on X
N <- 500            # sample size
beta0 <- 4.0        # true value
beta1 <- 1.5        # change it to 0 for testing
sigma2 <- 0.5       # compare with sigma2 = 4

# we consider the sampling distribution of the OLS estimators and t-test of H_0: beta1=0
# conditional on x. Hence this is fixed in repeated samples 
set.seed(123)         # for reproducibility
x <- rnorm(N)         # generate x ~ N(0, 1); alternatively use runif(N) for uniform(0,1)
x_bar <- mean(x)
dev_x <- sum((x - x_bar)^2)  # deviance of x
var_x <- dev_x / N           # var(x,1)
h <- 1/N + (x - x_bar)^2 / dev_x  # leverage
plot(x, h, pch=16, main="Leverage vs x", xlab="x", ylab="h")

# Monte Carlo Simulation
Repl <- 5000
vbeta0_hat <- numeric(Repl)
vbeta1_hat <- numeric(Repl)
vs2_hat <- numeric(Repl)
vt_test <- numeric(Repl)

for (j in 1:Repl) {
  eps <- sqrt(sigma2) * rnorm(N)  # generate epsilon
  y <- beta0 + beta1 * x + eps    # generate y
  y_bar <- mean(y)
  cov_xy <- sum((x - x_bar) * (y - y_bar)) / N  # cov(x,y)
  beta1_hat <- cov_xy / var_x
  beta0_hat <- y_bar - beta1_hat * x_bar
  y_hat <- beta0_hat + beta1_hat * x  # fitted values
  e <- y - y_hat                      # LS residuals
  s2_hat <- sum(e^2) / (N - 2)
  vbeta1_hat[j] <- beta1_hat
  vbeta0_hat[j] <- beta0_hat
  vs2_hat[j] <- s2_hat
  vt_test[j] <- beta1_hat / sqrt(s2_hat / dev_x)  # t-statistic
}

# Plot results
plot(x, y, pch=16, main="Last Simulated Sample", xlab="x", ylab="y")
lines(x, y_hat, col="blue")

hist(vbeta0_hat, freq=FALSE, main="Sampling Distribution of beta0_hat")
hist(vbeta1_hat, freq=FALSE, main="Sampling Distribution of beta1_hat")
hist(vs2_hat, freq=FALSE, main="Sampling Distribution of sigma^2_hat")

# t-statistic comparison
hist(vt_test, freq=FALSE, main="Sampling Distribution of t-statistic")
lines(sort(vt_test), dt(sort(vt_test), df=N-2), col="blue")  # comparison with Student's t pdf
